@@ -4,7 +4,7 @@ from __future__ import absolute_import
 import mock
 import pytest
 
-import django
+from django import VERSION as DJANGO_VERSION
 from django.contrib import messages
 from django.contrib.messages.middleware import MessageMiddleware
 from django.contrib.messages.storage.base import Message
@@ -20,7 +20,8 @@ from .compat import force_text
 from .factories import UserFactory
 from .helpers import TestViewHelper
 from .models import Article, CanonicalArticle
-from .views import (ArticleListView, AuthorDetailView, OrderableListView,
+from .views import (ArticleListView, ArticleListViewWithCustomQueryset,
+                    AuthorDetailView, OrderableListView,
                     FormMessagesView, ContextView)
 
 
@@ -182,6 +183,22 @@ class TestSelectRelatedMixin(TestViewHelper, test.TestCase):
         self.assertEqual(200, resp.status_code)
         m.assert_called_once_with('author')
 
+    @mock.patch('django.db.models.query.QuerySet.select_related')
+    def test_select_related_keeps_select_related_from_queryset(self, m):
+        """
+        Checks that an empty select_related attribute does not
+        cancel a select_related provided by queryset.
+        """
+        qs = Article.objects.all()
+        qs.select_related = m
+        m.reset_mock()
+
+        resp = self.dispatch_view(
+            self.build_request(),
+            view_class=ArticleListViewWithCustomQueryset)
+        self.assertEqual(200, resp.status_code)
+        self.assertEqual(0, m.call_count)
+
 
 class TestPrefetchRelatedMixin(TestViewHelper, test.TestCase):
     view_class = AuthorDetailView
@@ -218,6 +235,22 @@ class TestPrefetchRelatedMixin(TestViewHelper, test.TestCase):
         self.assertEqual(200, resp.status_code)
         m.assert_called_once_with('article_set')
 
+    @mock.patch('django.db.models.query.QuerySet.prefetch_related')
+    def test_prefetch_related_keeps_select_related_from_queryset(self, m):
+        """
+        Checks that an empty prefetch_related attribute does not
+        cancel a prefetch_related provided by queryset.
+        """
+        qs = Article.objects.all()
+        qs.prefetch_related = m
+        m.reset_mock()
+
+        resp = self.dispatch_view(
+            self.build_request(),
+            view_class=ArticleListViewWithCustomQueryset)
+        self.assertEqual(200, resp.status_code)
+        self.assertEqual(0, m.call_count)
+
 
 class TestOrderableListMixin(TestViewHelper, test.TestCase):
     view_class = OrderableListView
@@ -556,7 +589,7 @@ class MessageMixinTests(test.TestCase):
             self.get_request_response(TestView.as_view())
 
     @pytest.mark.skipif(
-        django.VERSION < (1, 5),
+        DJANGO_VERSION < (1, 5),
         reason='Some features of MessageMixin are only available in '
                'Django >= 1.5')
     def test_wrapper_available_in_dispatch(self):
@@ -585,7 +618,7 @@ class MessageMixinTests(test.TestCase):
         # This test is designed to break when django.contrib.messages.api
         # changes (items being added or removed).
         excluded_API = set()
-        if django.VERSION >= (1, 7):
+        if DJANGO_VERSION >= (1, 7):
             excluded_API.add('MessageFailure')
         self.assertEqual(
             _MessageAPIWrapper.API | excluded_API,
