@@ -12,6 +12,7 @@ from braces import views
 
 from .models import Article, CanonicalArticle
 from .forms import ArticleForm, FormWithUserKwarg
+from .helpers import SetJSONEncoder
 
 
 class OkView(View):
@@ -79,6 +80,18 @@ class SimpleJsonView(views.JSONResponseMixin, View):
         return self.render_json_response(object)
 
 
+class CustomJsonEncoderView(views.JSONResponseMixin, View):
+    """
+    A view for testing JSONResponseMixin's `json_encoder_class` attribute
+    with custom JSONEncoder class.
+    """
+    json_encoder_class = SetJSONEncoder
+
+    def get(self, request):
+        object = {'numbers': set([1, 2, 3])}
+        return self.render_json_response(object)
+
+
 class SimpleJsonBadRequestView(views.JSONResponseMixin, View):
     """
     A view for testing JSONResponseMixin's render_json_response() method with
@@ -135,6 +148,7 @@ class CreateArticleView(CreateView):
     """
     View for testing CreateAndRedirectEditToView.
     """
+    fields = ['author', 'title', 'body', 'slug']
     model = Article
     template_name = 'form.html'
 
@@ -174,6 +188,16 @@ class ArticleListView(views.SelectRelatedMixin, ListView):
     select_related = ('author',)
 
 
+class ArticleListViewWithCustomQueryset(views.SelectRelatedMixin, ListView):
+    """
+    Another list view for articles, required to test SelectRelatedMixin.
+    """
+    queryset = Article.objects.select_related('author').prefetch_related(
+        'article_set')
+    template_name = 'blank.html'
+    select_related = ()
+
+
 class FormWithUserKwargView(views.UserFormKwargsMixin, FormView):
     """
     View for testing UserFormKwargsMixin.
@@ -310,3 +334,13 @@ class UserPassesTestNotImplementedView(views.UserPassesTestMixin, OkView):
 class AllVerbsView(views.AllVerbsMixin, View):
     def all(self, request, *args, **kwargs):
         return HttpResponse('All verbs return this!')
+
+
+class SSLRequiredView(views.SSLRequiredMixin, OkView):
+    pass
+
+
+class RecentLoginRequiredView(views.RecentLoginRequiredMixin, OkView):
+    """
+    A view for testing RecentLoginRequiredMixin.
+    """
